import os
import re

__version__ = "0.2.3"

_STRTABLE = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
_HKEY = b"3AF0298C219469522A313570E8583005A642E73EDD58E3EA2FB7339D3DF1597E"

_KEYPARAMS = {
    "osName": "android",
    "osVersion": "6.0.1",
    "osLand": "ja_JP",
    "osTimezone": "Asia/Tokyo",
    "appId": "tv.abema",
    "appVersion": "3.27.1"
}

_MEDIATOKEN_API = "https://api.abema.io/v1/media/token"
_LICENSE_API = "https://license.abema.io/abematv-hls"
_USERAPI = "https://api.abema.io/v1/users"
_PROGRAMAPI = 'https://api.abema.io/v1/video/programs/'
_CHANNELAPI = 'https://api.abema.io/v1/media/slots/'

def is_channel(url):
    url = re.findall('(slots)', url)
    if url:
        return True
    return False


res_data = {
    "1080p": ["4000kb/s", "AAC 192kb/s 2ch"],
    "720p": ["2000kb/s", "AAC 160kb/s 2ch"],
    "480p": ["900kb/s", "AAC 128kb/s 2ch"],
    "360p": ["550kb/s", "AAC 128kb/s 2ch"],
    "240p": ["240kb/s", "AAC 64kb/s 1ch"],
    "180p": ["120kb/s", "AAC 64kb/s 1ch"]
}

"""
Admin Check Code from https://gist.github.com/sylvainpelissier/ff072a6759082590a4fe8f7e070a4952
"""
def isUserAdmin():
    if os.name == 'nt':
        import ctypes
        # WARNING: requires Windows XP SP2 or higher!
        try:
            return ctypes.windll.shell32.IsUserAnAdmin()
        except:
            print("Admin check failed, assuming not an admin.")
            return False
    else:
        return os.getuid() == 0



import hmac
import hashlib
import struct
import time
import uuid
import m3u8
import re

from base64 import urlsafe_b64encode
from binascii import unhexlify
from Crypto.Cipher import AES

from .common import _STRTABLE, _HKEY, _MEDIATOKEN_API, _LICENSE_API, _USERAPI, _KEYPARAMS, _PROGRAMAPI, _CHANNELAPI, is_channel

def get_auth_token(session, verbose):
    def key_secret(devid):
        SECRETKEY = (b"v+Gjs=25Aw5erR!J8ZuvRrCx*rGswhB&qdHd_SYerEWdU&a?3DzN9B"
                    b"Rbp5KwY4hEmcj5#fykMjJ=AuWz5GSMY-d@H7DMEh3M@9n2G552Us$$"
                    b"k9cD=3TxwWe86!x#Zyhe")
        deviceid = devid.encode("utf-8")
        ts_1hour = (int(time.time()) + 60 * 60) // 3600 * 3600
        time_struct = time.gmtime(ts_1hour)
        ts_1hour_str = str(ts_1hour).encode("utf-8")

        h = hmac.new(SECRETKEY, digestmod=hashlib.sha256)
        h.update(SECRETKEY)
        tmp = h.digest()

        for i in range(time_struct.tm_mon):
            h = hmac.new(SECRETKEY, digestmod=hashlib.sha256)
            h.update(tmp)
            tmp = h.digest()

        h = hmac.new(SECRETKEY, digestmod=hashlib.sha256)
        h.update(urlsafe_b64encode(tmp).rstrip(b"=") + deviceid)
        tmp = h.digest()

        for i in range(time_struct.tm_mday % 5):
            h = hmac.new(SECRETKEY, digestmod=hashlib.sha256)
            h.update(tmp)
            tmp = h.digest()

        h = hmac.new(SECRETKEY, digestmod=hashlib.sha256)
        h.update(urlsafe_b64encode(tmp).rstrip(b"=") + ts_1hour_str)
        tmp = h.digest()

        for i in range(time_struct.tm_hour % 5):  # utc hour
            h = hmac.new(SECRETKEY, digestmod=hashlib.sha256)
            h.update(tmp)
            tmp = h.digest()

        finalize = urlsafe_b64encode(tmp).rstrip(b"=").decode("utf-8")

        if verbose:
            print('[DEBUG] Secret Key: {}'.format(finalize))

        return finalize

    deviceid = str(uuid.uuid4())
    if verbose:
        print('[DEBUG] Generated Device UUID: {}'.format(deviceid))
    json_data = {"deviceId": deviceid, "applicationKeySecret": key_secret(deviceid)}

    if verbose:
        print('[DEBUG] Sending json data')
    res = session.post(_USERAPI, json=json_data).json()

    try:
        if verbose:
            print('[DEBUG] Data sended, getting token')
        token = res['token']
        if verbose:
            print('[DEBUG] Usertoken: {}'.format(token))
    except:
        print('[ERROR] Failed to get usertoken')
        exit(1)

    return ['bearer ' + token, deviceid]


def fetch_video_key(ticket=None, authToken=None, session=None, verbose=False):
    if verbose:
        print('[DEBUG] Sending parameter to API')
    restoken = session.get(_MEDIATOKEN_API, params=_KEYPARAMS).json()
    mediatoken = restoken['token']
    if verbose:
        print('[DEBUG] Mediatoken: {}'.format(mediatoken))

    if verbose:
        print('[DEBUG] Sending ticket and mediatoken to License API')
    rgl = session.post(_LICENSE_API, params={"t": mediatoken}, json={"kv": "a", "lt": ticket})
    if rgl.status_code == 403:
        print('[ERROR] Access to the video are not allowed\nProbably a premium video or geo-locked.')
        if verbose:
            print('[ERROR] Code 403: ' + rgl.text)
        exit(1)

    gl = rgl.json()

    cid = gl['cid']
    k = gl['k']

    if verbose:
        print('[DEBUG] CID: {}'.format(cid))
        print('[DEBUG] K: {}'.format(k))

    if verbose:
        print('[DEBUG] Summing up data with STRTABLE')
    res = sum([_STRTABLE.find(k[i]) * (58 ** (len(k) - 1 - i)) for i in range(len(k))])

    if verbose:
        print('[DEBUG] Result: {}'.format(res))
        print('[DEBUG] Intepreting data')

    encvk = struct.pack('>QQ', res >> 64, res & 0xffffffffffffffff)

    if verbose:
        print('[DEBUG] Encoded video key: {}'.format(encvk))
        print('[DEBUG] Hashing data')

    h = hmac.new(unhexlify(_HKEY), (cid + authToken[1]).encode("utf-8"), digestmod=hashlib.sha256)
    enckey = h.digest()

    if verbose:
        print('[DEBUG] Second Encoded video key: {}'.format(enckey))
        print('[DEBUG] Decrypting result')

    aes = AES.new(enckey, AES.MODE_ECB)
    vkey = aes.decrypt(encvk)

    if verbose:
        print('[DEBUG] Decrypted, Resulting output: {}'.format(vkey))

    return vkey


def parsem3u8(hls, res, session, verbose):
    if verbose:
        print('[DEBUG] Requesting m3u8')
    r = session.get(hls)
    if verbose and r.status_code == 200:
        if r.status_code == 200:
            print('[DEBUG] m3u8 requested')
            print('[DEBUG] Parsing m3u8')
        elif r.status_code == 403:
            print('[DEBUG] Forbidden access to the m3u8 url')
            print('[DEBUG] Probably a premium video.')
    if r.status_code == 403:
        print('[ERROR] Video are geo-locked to Japanese only.')
        if verbose:
            print('[ERROR] Code 403: ' + r.text)
        exit(1)
    x = m3u8.loads(r.text)
    files = x.files
    resgex = re.findall(r'(\d*)(?:\/\w+.ts)', files[1])[0]
    iv = x.keys[0].iv
    ticket = x.keys[0].uri[18:]
    if res[:-1] != resgex:
        print('[WARN] Resolution {} are not available'.format(res))
        print('[WARN] Switching to {}p'.format(resgex))
    if verbose:
        print('[DEBUG] Total files: {}'.format(len(files[1:])))
        print('[DEBUG] IV: {}'.format(iv))
        print('[DEBUG] Ticket key: {}'.format(ticket))
    return [files[1:], iv[2:], ticket]


def available_resolution(m3u8_, session, verbose):
    if verbose:
        print('[DEBUG] Requesting data to API')

    m3u8_ = m3u8_[:m3u8_.rfind('/')]
    m3u8_1080 = m3u8_[:m3u8_.rfind('/')] + '/1080/playlist.m3u8'
    m3u8_720 = m3u8_[:m3u8_.rfind('/')] + '/720/playlist.m3u8'
    m3u8_480 = m3u8_[:m3u8_.rfind('/')] + '/480/playlist.m3u8'
    m3u8_360 = m3u8_[:m3u8_.rfind('/')] + '/360/playlist.m3u8'
    m3u8_240 = m3u8_[:m3u8_.rfind('/')] + '/240/playlist.m3u8'
    m3u8_180 = m3u8_[:m3u8_.rfind('/')] + '/180/playlist.m3u8'

    rr_all = session.get(m3u8_[:m3u8_.rfind('/')] + '/playlist.m3u8').text
    r_all = m3u8.loads(rr_all)
    r1080 = m3u8.loads(session.get(m3u8_1080).text)
    r720 = m3u8.loads(session.get(m3u8_720).text)
    r480 = m3u8.loads(session.get(m3u8_480).text)
    r360 = m3u8.loads(session.get(m3u8_360).text)
    r240 = m3u8.loads(session.get(m3u8_240).text)
    r180 = m3u8.loads(session.get(m3u8_180).text)

    play_res = []
    for r_p in r_all.playlists:
        play_res.append(list(r_p.stream_info.resolution))

    x1080 = r1080.files[1:][0]
    x720 = r720.files[1:][0]
    x480 = r480.files[1:][0]
    x360 = r360.files[1:][0]
    x240 = r240.files[1:][0]
    x180 = r180.files[1:][0]

    resgex = re.compile(r'(\d*)(?:\/\w+.ts)')

    ava_reso = []
    if '1080' in re.findall(resgex, x1080):
        temp_ = ['1080p']
        for r in play_res:
            if 1080 in r:
                temp_.append('{w}x{h}'.format(w=r[0], h=r[1]))
        ava_reso.append(temp_)
    if '720' in re.findall(resgex, x720):
        temp_ = ['720p']
        for r in play_res:
            if 720 in r:
                temp_.append('{w}x{h}'.format(w=r[0], h=r[1]))
        ava_reso.append(temp_)
    if '480' in re.findall(resgex, x480):
        temp_ = ['480p']
        for r in play_res:
            if 480 in r:
                temp_.append('{w}x{h}'.format(w=r[0], h=r[1]))
        ava_reso.append(temp_)
    if '360' in re.findall(resgex, x360):
        temp_ = ['360p']
        for r in play_res:
            if 360 in r:
                temp_.append('{w}x{h}'.format(w=r[0], h=r[1]))
        ava_reso.append(temp_)
    if '240' in re.findall(resgex, x240):
        temp_ = ['240p']
        for r in play_res:
            if 240 in r:
                temp_.append('{w}x{h}'.format(w=r[0], h=r[1]))
        ava_reso.append(temp_)
    if '180' in re.findall(resgex, x180):
        temp_ = ['180p']
        for r in play_res:
            if 180 in r:
                temp_.append('{w}x{h}'.format(w=r[0], h=r[1]))
        ava_reso.append(temp_)

    return ava_reso


def webparse_m3u8(m3u8, session, verbose):
    if verbose:
        print('[DEBUG] Requesting data to API')
    reg = re.compile(r'(program|slot)\/[\w+-]+')
    res = re.search(reg, m3u8)[0]
    eplink = res[res.find('/')+1:]

    if 'slot' in res:
        req = session.get(_CHANNELAPI + eplink)
        if verbose and req.status_code == 200:
            print('[DEBUG] Data requested')
            print('[DEBUG] Parsing json API')
        jsdata = req.json()
        title = jsdata['slot']['title']
        res = m3u8[:m3u8.rfind('/')]
        res = res[res.rfind('/')+1:]

        if verbose:
            print('[DEBUG] M3U8 Link: {}'.format(m3u8))
            print('[DEBUG] Title: {}'.format(title))

        return title, res
    else:
        req = session.get(_PROGRAMAPI + eplink)
        if verbose and req.status_code == 200:
            print('[DEBUG] Data requested')
            print('[DEBUG] Parsing json API')
        jsdata = req.json()
        title = jsdata['series']['title']
        epnum = jsdata['episode']['title']
        outputfile = title + ' - ' + epnum

        res = m3u8[:m3u8.rfind('/')]
        res = res[res.rfind('/')+1:]

        if verbose:
            print('[DEBUG] M3U8 Link: {}'.format(m3u8))
            print('[DEBUG] Video title: {}'.format(title))
            print('[DEBUG] Episode number: {}'.format(epnum))

        return outputfile, res


def webparse(url, res, session, verbose):
    if verbose:
        print('[DEBUG] Requesting data to API')
    eplink = url[url.rfind('/')+1:]

    if is_channel(url):
        req = session.get(_CHANNELAPI + eplink)
        if verbose and req.status_code == 200:
            print('[DEBUG] Data requested')
            print('[DEBUG] Parsing json API')
        jsdata = req.json()
        title = jsdata['slot']['title']
        if 'chasePlayback' in jsdata['slot']: # just in case
            hls = jsdata['slot']['chasePlayback']['hls']
        else:
            hls = jsdata['slot']['playback']['hls']

        m3u8link = '{x}/{r}/playlist.m3u8'.format(x=hls[:hls.rfind('/')], r=res[:-1])

        if verbose:
            print('[DEBUG] M3U8 Link: {}'.format(m3u8link))
            print('[DEBUG] Title: {}'.format(title))

        return title, m3u8link
    else:
        req = session.get(_PROGRAMAPI + eplink)
        if verbose and req.status_code == 200:
            print('[DEBUG] Data requested')
            print('[DEBUG] Parsing json API')
        jsdata = req.json()
        title = jsdata['series']['title']
        epnum = jsdata['episode']['title']
        hls = jsdata['playback']['hls']
        outputfile = title + ' - ' + epnum

        m3u8link = '{x}/{r}/playlist.m3u8'.format(x=hls[:hls.rfind('/')], r=res[:-1])

        if verbose:
            print('[DEBUG] M3U8 Link: {}'.format(m3u8link))
            print('[DEBUG] Video title: {}'.format(title))
            print('[DEBUG] Episode number: {}'.format(epnum))

        return outputfile, m3u8link



import os
import shutil
import subprocess

import click
import requests

from .common import __version__, isUserAdmin, res_data
from .downloader import get_video, merge_video
from .parser import (available_resolution, fetch_video_key, get_auth_token,
                     parsem3u8, webparse, webparse_m3u8)

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'], ignore_unknown_options=True)

@click.group(context_settings=CONTEXT_SETTINGS, invoke_without_command=True)
@click.option('--version', '-V', is_flag=True, help="Show current version")
@click.option('--update', '-U', is_flag=True, help="Update yuu to the newest version")
def cli(version=False, update=False):
    """
    A simple AbemaTV video downloader
    """
    if version:
        print('yuu v{} - Created by NoAiOne'.format(__version__))
        exit(0)
    if update:
        print('[INFO] Updating yuu...'.format(ver=__version__))
        upstream_data = requests.get("https://pastebin.com/raw/Bt3ZLjfu").json()
        upstream_version = upstream_data['version']
        upstream_change = upstream_data['changelog']
        if upstream_version == __version__:
            print('[INFO] Already on the newest version.')
            exit(0)
        print('[INFO] Updating to yuu version {} (Current: v{})'.format(upstream_version, __version__))

        try:
            if isUserAdmin():
                if os.name != "nt":
                    print('[WARN] Run this command to update yuu: `pip3 install -U yuu=={}`'.format(upstream_version))
                    exit(1)
                from win32com.shell.shell import ShellExecuteEx
                ShellExecuteEx(lpVerb='runas', lpFile='pip', lpParameters='install -U yuu=={}'.format(upstream_version))
            else:
                print('[WARN] Run this command to update yuu: `pip install -U yuu=={}`'.format(upstream_version))
                exit(1)
        except:
            print('[ERROR] Updater returned non-zero exit code')
            print('Try to run `pip install -U yuu={}` manually'.format(upstream_version))
            exit(1)
        print('\n=== yuu version {} changelog ==='.format(upstream_version))
        print(upstream_change+'\n')
        print('[INFO] Updated.')
        exit(0)


@cli.command("download", short_help="Download video from abema.tv")
@click.argument("input", metavar="<AbemaTV url site or m3u8>")
@click.option("--proxy", "-p", required=False, default=None, metavar="<ip:port/url>", help="Use http(s)/socks5 proxies (please add `socks5://` if you use socks5)")
@click.option("--resolution", "-r", "res", required=False, type=click.Choice(['180p', '240p', '360p', '480p', '720p', '1080p']), default="1080p", help="Resolution to be downloaded (Default: 1080p)")
@click.option("--resolutions", "-R", "resR", is_flag=True, help="Show available resolutions")
@click.option("--output", "-o", required=False, default=None, help="Output filename")
@click.option('--verbose', '-v', is_flag=True, help="Enable verbosity")
def main_downloader(input, proxy, res, resR, output, verbose):
    """Download a free video from abema"""
    print('[INFO] Starting yuu v{ver}...'.format(ver=__version__))

    upstream_data = requests.get("https://pastebin.com/raw/Bt3ZLjfu").json()
    upstream_version = upstream_data['version']
    if upstream_version != __version__:
        print('[INFO] There\'s new version available to download, please update using `yuu -U`.')
        exit(0)

    sesi = requests.Session()
    if proxy:
        print('[INFO] Testing proxy')
        sesi.proxies = {'http': proxy, 'https': proxy}
        # Somebody tell me how to do recursive test properly
        try:
            if verbose:
                print('[DEBUG] Testing http+https mode proxy')
            sesi.get('http://httpbin.org/get') # Some test website to check if proxy works or not
            pmode = "HTTP+HTTPS/SOCKS5"
        except requests.exceptions.RequestException:
            if verbose:
                print('[DEBUG] Failed')
            sesi = requests.Session()
            sesi.proxies = {'http': proxy}
            try:
                if verbose:
                    print('[DEBUG] Testing http mode proxy')
                sesi.get('http://httpbin.org/get') # This too but in https mode
                pmode = "HTTP/SOCKS5"
            except requests.exceptions.RequestException:
                if verbose:
                    print('[DEBUG] Failed')
                sesi = requests.Session()
                sesi.proxies = {'https': proxy} # Final test if it's failed then it will return error
                try:
                    if verbose:
                        print('[DEBUG] Testing https mode proxy')
                    sesi.get('http://httpbin.org/get')
                    pmode = "HTTPS/SOCKS5"
                except requests.exceptions.RequestException:
                    if verbose:
                        print('[DEBUG] Failed')
                    print('[ERROR] Cannot connect to proxy (Request timeout)')
                    exit(1)
    try:
        sesi.get('http://httpbin.org/get')
        pmode = "No proxy"
    except:
        print('[ERROR] No connection available to make requests')
        exit(1)
    if verbose:
        print('[DEBUG] Using proxy mode: {}'.format(pmode))

    print('[INFO] Fetching user token')
    authtoken = get_auth_token(sesi, verbose)
    sesi.headers.update({'Authorization': authtoken[0]})

    if input[-5:] != '.m3u8':
        print('[INFO] Parsing webpage')
        outputtitle, m3u8link = webparse(input, res, sesi, verbose)
        if resR:
            print('[INFO] Checking available resolution')
            avares = available_resolution(m3u8link, sesi, verbose)
            print('[INFO] Available resolution:')
            print('{0: <{width}}{1: <{width}}{2: <{width}}{3: <{width}}'.format("", "Resolution", "Video Quality", "Audio Quality", width=16))
            for res in avares:
                r_c, wxh = res
                vidq, audq = res_data[r_c]
                print('{0: <{width}}{1: <{width}}{2: <{width}}{3: <{width}}'.format('>> ' + r_c, wxh, vidq, audq, width=16))
            exit(0)
        print('[INFO] Parsing m3u8')
        files, iv, ticket = parsem3u8(m3u8link, res, sesi, verbose)
        if output:
            if output[-3:] == '.ts':
                output = output
            else:
                output = output + '.ts'
        else:
            output = '{x} (AbemaTV {r}).ts'.format(x=outputtitle, r=res)
        if verbose:
            print('[DEBUG] Output file: {}'.format(output))
    else:
        print('[INFO] Parsing m3u8')
        outputtitle, res = webparse_m3u8(input, sesi, verbose)
        files, iv, ticket = parsem3u8(input, res, sesi, verbose)
        if output:
            if output[-3:] == '.ts':
                output = output
            else:
                output = output + '.ts'
        else:
            output = '{x} (AbemaTV {r}).ts'.format(x=outputtitle, r=res)

    # Don't use forbidden/illegal character (replace it with underscore)
    illegalchar = ['/', '<', '>', ':', '"', '\\', '|', '?', '*'] # https://docs.microsoft.com/en-us/windows/desktop/FileIO/naming-a-file
    for char in illegalchar:
        output = output.replace(char, '_')

    print('[INFO] Fetching video key')
    getkey = fetch_video_key(ticket, authtoken, sesi, verbose)
    
    print('[INFO][DOWN] Starting downloader...')
    print('[INFO][DOWN] Resolution: {}'.format(res))
    dllist, tempdir = get_video(files, getkey, iv, sesi, verbose)
    if not dllist:
        if tempdir:
            shutil.rmtree(tempdir)
        exit(0)
    print('[INFO][DOWN] Finished downloading')
    print('[INFO] Merging video')
    merge_video(dllist, output)

    print('[INFO] Cleaning up')
    shutil.rmtree(tempdir)
    exit(0)


def main():
    cli()


if __name__=='__main__':
    cli()



import os
import tempfile

from binascii import unhexlify
from Crypto.Cipher import AES
from tqdm import tqdm

def decrypt_data(tsdata, key, iv):
    def _decrypt(d, k, iv):
        dec = AES.new(k, AES.MODE_CBC, IV=iv)
        return dec.decrypt(d)

    if iv.startswith('0x'):
        iv = iv[2:]
    iv = unhexlify(iv)

    return _decrypt(tsdata, key, iv)


def get_video(fileslist, key, iv, session, verbose):
    print('[INFO][DOWN] Creating temporary folder')
    tempdir = tempfile.mkdtemp()
    dledfiles = []

    try:
        if not verbose:
            with tqdm(total=len(fileslist), desc='Downloading', ascii=True, unit='file') as pbar:
                for tsf in fileslist:
                    outputtemp = os.path.basename(tsf)
                    if outputtemp.find('?tver') != -1:
                        outputtemp = outputtemp[:outputtemp.find('?tver')]
                    outputtemp = tempdir + '\\' + outputtemp
                    with open(outputtemp, 'wb') as outf:
                        try:
                            req = session.get(tsf)
                            outf.write(decrypt_data(req.content, key, iv))
                        except Exception as err:
                            print('[ERROR] Problem occured\nreason: {}'.format(err))
                            exit(1)
                    pbar.update()
                    dledfiles.append(outputtemp)
        elif verbose:
            for tsf in fileslist:
                outputtemp = os.path.basename(tsf)
                if outputtemp.find('?tver') != -1:
                    outputtemp = outputtemp[:outputtemp.find('?tver')]
                otpt = outputtemp
                outputtemp = os.path.join(tempdir, outputtemp)
                with open(outputtemp, 'wb') as outf:
                    try:
                        print('[DEBUG][DOWN] Requesting & decrypting content for: {}'.format(otpt))
                        req = session.get(tsf)
                        outf.write(decrypt_data(req.content, key, iv))
                    except Exception as err:
                        print('[ERROR] Problem occured\nreason: {}'.format(err))
                        exit(1)
                dledfiles.append(outputtemp)
    except KeyboardInterrupt:
        print('[WARN] User pressed CTRL+C, cleaning up...')
        return None, tempdir

    return dledfiles, tempdir


def merge_video(inp, out):
    with open(out, 'wb') as outf:
        with tqdm(total=len(inp), desc='Merging', ascii=True, unit='file') as pbar:
            for i in inp:
                with open(i, 'rb') as c:
                    outf.write(c.read())
                os.remove(i)
                pbar.update()



import setuptools
from yuu.common import __version__

with open('README.md', 'r') as f:
    desc = f.read()

setuptools.setup(
    name = 'yuu',
    version = __version__,
    description = 'Yuu - A simple AbemaTV video downloader',
    long_description = desc,
    long_description_content_type = "text/markdown",
    author = 'noaione',
    author_email = 'noaione0809@gmail.com',
    keywords = [
        'ripping', 
        'downloader', 
        'parser'
    ],
    license = 'BSD-3-Clause',
    url = 'https://github.com/noaione/yuu',
    packages = setuptools.find_packages(),
    install_requires = [
        'requests[socks]', 
        'm3u8', 
        'tqdm', 
        'pycryptodome',
        'click'
    ],
    classifiers = [
        'Development Status :: 5 - Production/Stable', 
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 3', 
        'Programming Language :: Python :: 3.5', 
        'Programming Language :: Python :: 3.6', 
        'Programming Language :: Python :: 3.7'
    ],
    entry_points = {
        'console_scripts': ['yuu=yuu.command:main']
    }
)